@@ -29,31 +29,47 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-inet-connection.h>
 
+#ifdef USE_LIBGPS
+#include <gps.h>
+#endif
+
 #ifndef NAN
 #define NAN (0.0/0.0)
 #endif /* !NAN */
 
+#define MAXCHANNELS 20
 #define MAXTAGLEN    8       /* maximum length of sentence tag name */
 
-struct gps_fix_t {
+typedef struct {
+  int prn;     /* prn of satellite, -1 if no info */
+  int ss;      /* signal to noise ratio */
+  int used;    /* satellite is is being used */
+} gps_sat_t;
+
+struct gps_t {
   double latitude;	/* Latitude in degrees */
   double longitude;	/* Longitude in degrees */
   double altitude;      /* Altitude in meters */
   double eph;  	        /* Horizontal position uncertainty, meters */
   double track;	        /* Course made good (relative to true north) */
+
+  int sat_num;
+  gps_sat_t sat_data[MAXCHANNELS];
 };
 
 typedef unsigned int gps_mask_t;
 
-#define LATLON_SET	 (1<<0)
-#define ALTITUDE_SET	 (1<<1)
-#define TRACK_SET	 (1<<2)
-#define HERR_SET	 (1<<3)
+#define FIX_LATLON_SET	   (1<<0)
+#define FIX_ALTITUDE_SET   (1<<1)
+#define FIX_TRACK_SET	   (1<<2)
+#define FIX_HERR_SET	   (1<<3)
+#define FIX_SATELLITE_SET  (1<<4)
 
-#define LATLON_CHANGED   (LATLON_SET   << 8)
-#define ALTITUDE_CHANGED (ALTITUDE_SET << 8)
-#define TRACK_CHANGED    (TRACK_SET    << 8)
-#define HERR_CHANGED     (HERR_SET     << 8)
+#define LATLON_CHANGED    (FIX_LATLON_SET    << 8)
+#define ALTITUDE_CHANGED  (FIX_ALTITUDE_SET  << 8)
+#define TRACK_CHANGED     (FIX_TRACK_SET     << 8)
+#define HERR_CHANGED      (FIX_HERR_SET      << 8)
+#define SATELLITE_CHANGED (FIX_SATELLITE_SET << 8)
 
 #define CHANGED_MASK 0xff00
 
@@ -65,7 +81,7 @@ typedef unsigned int gps_mask_t;
 #include <errno.h>
 #endif
 
-typedef void (*gps_cb)(gps_mask_t set, struct gps_fix_t *fix, void *data);
+typedef void (*gps_cb)(gps_mask_t set, struct gps_t *fix, void *data);
 #define GPS_CB(f) ((gps_cb)(f))
 
 typedef struct {
@@ -86,22 +102,28 @@ typedef struct gps_state {
 #endif
 
   GThread* thread_p;
-  GMutex *mutex;
+  GMutex *mutex, *control_mutex;
+
+#ifdef USE_LIBGPS
+  struct gps_data_t *data;
+#else
   GnomeVFSInetConnection *iconn;
   GnomeVFSSocket *socket;
+#endif
 
 #else
   LocationGPSDevice *device;
   LocationGPSDControl *control;
+  gboolean connected;
   guint idd_changed;
 #endif
 
   struct {
-    struct gps_fix_t fix;
+    struct gps_t fix;
     gps_mask_t set;
   } last;
 
-  struct gps_fix_t fix;   
+  struct gps_t fix;   
   gps_mask_t set;
 
   GSList *callbacks;
