@@ -45,6 +45,14 @@ static OsmGpsMapSource_t map_sources[] = {
 };
 static int num_map_sources = sizeof(map_sources)/sizeof(map_sources[0]);
 
+#ifdef OSD_DOUBLEPIXEL
+#define OSD_DPIX_EXTRA 1
+#define OSD_DPIX_SKIP  (OSD_FONT_SIZE/2)
+#else
+#define OSD_DPIX_EXTRA 0
+#define OSD_DPIX_SKIP  0
+#endif
+
 //the osd controls
 typedef struct {
     /* the offscreen representation of the OSD */
@@ -96,6 +104,14 @@ typedef struct {
     } nav;
 #endif
 
+#ifdef OSD_HEARTRATE
+    struct {
+        cairo_surface_t *surface;
+        gint rate;
+        gboolean ok;
+    } hr;
+#endif
+
 #ifdef OSD_COORDINATES
     struct {
         cairo_surface_t *surface;
@@ -112,6 +128,7 @@ typedef struct {
         gint handler_id;
         gint width, height;
         gboolean rendered;
+        gint max_h;
     } source_sel;
 #endif
 
@@ -553,7 +570,7 @@ osd_shape(cairo_t *cr, GdkColor *bg, GdkColor *fg) {
 #else
 static void 
 osd_shape(cairo_t *cr) {
-    cairo_set_source_rgb (cr, OSD_COLOR_BG);
+    cairo_set_source_rgba (cr, OSD_COLOR_BG);
     cairo_fill_preserve (cr);
     cairo_set_source_rgb (cr, OSD_COLOR);
     cairo_set_line_width (cr, 1);
@@ -717,16 +734,65 @@ osd_source_content(osm_gps_map_osd_t *osd, cairo_t *cr, gint offset) {
                                     CAIRO_FONT_WEIGHT_BOLD);
             cairo_set_font_size (cr, OSD_FONT_SIZE);
 
-            int i, step = (priv->source_sel.height - 2*OSD_TEXT_BORDER) / 
-                num_map_sources;
-            for(i=0;i<num_map_sources;i++) {
+            int i, step = (priv->source_sel.height - 2*OSD_TEXT_BORDER - OSD_DPIX_SKIP) / 
+                (num_map_sources + OSD_DPIX_EXTRA);
+            for(i=0;i<num_map_sources + OSD_DPIX_EXTRA;i++) {
                 cairo_text_extents_t extents;
-                const char *src = osm_gps_map_source_get_friendly_name( map_sources[i] );
+                const char *src = NULL;
+#if OSD_DPIX_EXTRA > 0
+                /* draw "double pixel" check button */
+                const char *dpix = _("Double Pixel");
+                if(i == num_map_sources) {
+                    src = dpix;
+                } else
+#endif
+                src = osm_gps_map_source_get_friendly_name( map_sources[i] );
+
                 cairo_text_extents (cr, src, &extents);
                 
                 int x = offset + OSD_S_PW + OSD_TEXT_BORDER;
                 int y = offset + step * i + OSD_TEXT_BORDER;
 
+#if OSD_DPIX_EXTRA > 0
+                if(i == num_map_sources) {
+                    int skip3 = OSD_DPIX_SKIP/3;
+
+                    gboolean dpix;
+                    g_object_get(osd->widget, "double-pixel", &dpix, NULL);
+
+                    cairo_set_line_width (cr, skip3);
+
+                    /* draw seperator line */
+                    cairo_move_to (cr, x, y + skip3);
+                    cairo_rel_line_to (cr, OSD_S_AREA_W - 2*OSD_TEXT_BORDER, 0);
+                    cairo_stroke(cr);
+
+                    y += OSD_DPIX_SKIP;
+
+                    /* draw check box */
+                    cairo_move_to (cr, x, y);
+                    cairo_rel_line_to (cr, OSD_FONT_SIZE, 0);
+                    cairo_rel_line_to (cr, 0, OSD_FONT_SIZE);
+                    cairo_rel_line_to (cr, -OSD_FONT_SIZE, 0);
+                    cairo_rel_line_to (cr, 0, -OSD_FONT_SIZE);
+                    cairo_stroke(cr);
+
+                    /* draw check if needed */
+                    if(dpix) {
+                        int space = OSD_FONT_SIZE/4;
+                        cairo_move_to (cr, x + space, y + space);
+                        cairo_rel_line_to (cr, OSD_FONT_SIZE - 2*space, OSD_FONT_SIZE - 2*space);
+                        cairo_stroke(cr);
+
+                        cairo_move_to (cr, x + OSD_FONT_SIZE - space, y + space);
+                        cairo_rel_line_to (cr, -(OSD_FONT_SIZE - 2*space), OSD_FONT_SIZE - 2*space);
+                        cairo_stroke(cr);
+                    }
+
+                    x += 1.5 * OSD_FONT_SIZE;
+                }
+#endif
+
                 /* draw filled rectangle if selected */
                 if(source == map_sources[i]) {
                     cairo_rectangle(cr, x - OSD_TEXT_BORDER/2, 
@@ -740,7 +806,7 @@ osd_source_content(osm_gps_map_osd_t *osd, cairo_t *cr, gint offset) {
                     GdkColor bg = osd->widget->style->bg[GTK_STATE_NORMAL];
                     gdk_cairo_set_source_color(cr, &bg);
 #else
-                    cairo_set_source_rgb (cr, OSD_COLOR_BG);
+                    cairo_set_source_rgba (cr, OSD_COLOR_BG);
 #endif
                 }
 
@@ -835,19 +901,20 @@ osd_source_reallocate(osm_gps_map_osd_t *osd) {
         cairo_set_font_size (cr, OSD_FONT_SIZE);
 
         /* calculate menu size */
-        int i, max_h = 0, max_w = 0;
+        int i,  max_w = 0;
+        priv->source_sel.max_h = 0;
         for(i=0;i<num_map_sources;i++) {
             const char *src = osm_gps_map_source_get_friendly_name( map_sources[i] );
             cairo_text_extents (cr, src, &extents);
 
             if(extents.width > max_w) max_w = extents.width;
-            if(extents.height > max_h) max_h = extents.height;
+            if(extents.height > priv->source_sel.max_h) priv->source_sel.max_h = extents.height;
         }
         cairo_destroy(cr);
        
         priv->source_sel.width  = max_w + 2*OSD_TEXT_BORDER;
-        priv->source_sel.height = num_map_sources * 
-            (max_h + 2*OSD_TEXT_SKIP) + 2*OSD_TEXT_BORDER;
+        priv->source_sel.height = (num_map_sources + OSD_DPIX_EXTRA) * 
+            (priv->source_sel.max_h + 2*OSD_TEXT_SKIP) + 2*OSD_TEXT_BORDER + OSD_DPIX_SKIP;
 
         w = OSD_S_EXP_W;
         h = OSD_S_EXP_H;
@@ -976,25 +1043,42 @@ osd_source_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
            y > 0 &&
            y < OSD_S_EXP_H) {
             
-            int step = (priv->source_sel.height - 2*OSD_TEXT_BORDER) 
-                / num_map_sources;
+            int step = (priv->source_sel.height - 2*OSD_TEXT_BORDER - OSD_DPIX_SKIP) 
+                / (num_map_sources + OSD_DPIX_EXTRA);
 
             y -= OSD_TEXT_BORDER - OSD_TEXT_SKIP;
-            y /= step;
+            int py = y / step;
 
             if(down) {
                 gint old = 0;
                 g_object_get(osd->widget, "map-source", &old, NULL);
 
-                if(y >= 0 &&
-                   y < num_map_sources &&
-                   old != map_sources[y]) {
-                    g_object_set(osd->widget, "map-source", map_sources[y], NULL);
+                if(py >= 0 &&
+                   py < num_map_sources &&
+                   old != map_sources[py]) {
+                    g_object_set(osd->widget, "map-source", map_sources[py], NULL);
                     
                     osd_render_source_sel(osd, TRUE);
                     osm_gps_map_repaint(OSM_GPS_MAP(osd->widget));
                     osm_gps_map_redraw(OSM_GPS_MAP(osd->widget));
                 }
+
+#if OSD_DPIX_EXTRA > 0
+                if(py >= num_map_sources) {
+                    y -= num_map_sources * (priv->source_sel.max_h + 2*OSD_TEXT_SKIP ) +
+                        OSD_TEXT_SKIP + OSD_DPIX_SKIP;
+
+                    if(y >= 0) {
+                        gboolean dpix = 0;
+                        g_object_get(osd->widget, "double-pixel", &dpix, NULL);
+                        g_object_set(osd->widget, "double-pixel", !dpix, NULL);
+                    
+                        osd_render_source_sel(osd, TRUE);
+                        osm_gps_map_repaint(OSM_GPS_MAP(osd->widget));
+                        osm_gps_map_redraw(OSM_GPS_MAP(osd->widget));
+                    }
+                }
+#endif
             }
             
             /* return "clicked in OSD background" to prevent further */
@@ -1463,6 +1547,132 @@ osm_gps_map_osd_draw_nav (OsmGpsMap *map, gboolean imperial,
 
 #endif // OSD_NAV
 
+#ifdef OSD_HEARTRATE
+#ifndef OSD_HR_FONT_SIZE
+#define OSD_HR_FONT_SIZE 60
+#endif
+#define OSD_HR_W  3*OSD_HR_FONT_SIZE
+#define OSD_HR_H  OSD_HR_FONT_SIZE
+
+#ifndef OSD_HR_Y
+#define OSD_HR_Y OSD_Y
+#endif
+
+static void
+osd_render_heart_shape(cairo_t *cr, gint x, gint y, gint s) {
+    cairo_move_to (cr, x-s, y-s/4);
+    cairo_curve_to (cr, x-s, y-s,     x,   y-s,     x,   y-s/4);
+    cairo_curve_to (cr, x,   y-s,     x+s, y-s,     x+s, y-s/4);
+    cairo_curve_to (cr, x+s, y+s/2,   x,   y+3*s/4, x,   y+s);
+    cairo_curve_to (cr, x,   y+3*s/4, x-s, y+s/2,   x-s, y-s/4);
+}
+
+static void 
+osd_render_heart(cairo_t *cr, gint x, gint y, gint s, gboolean ok) {
+    /* xyz */
+
+    cairo_set_line_join (cr, CAIRO_LINE_JOIN_ROUND);
+
+    osd_render_heart_shape(cr, x, y, s);
+    cairo_set_source_rgb (cr, 1, 1, 1);
+    cairo_set_line_width (cr, s/2);
+    cairo_stroke(cr);
+
+    osd_render_heart_shape(cr, x, y, s);
+    if(ok) cairo_set_source_rgb (cr, 1, 0, 0);
+    else   cairo_set_source_rgb (cr, 0.8, 0.8, 0.8);
+    cairo_fill_preserve (cr);
+    cairo_set_line_width (cr, s/5);
+
+    cairo_stroke(cr);
+}
+
+static void
+osd_render_hr(osm_gps_map_osd_t *osd) 
+{
+    osd_priv_t *priv = (osd_priv_t*)osd->priv; 
+
+    if(!priv->hr.surface)
+        return;
+
+    /* first fill with transparency */
+    g_assert(priv->hr.surface);
+    cairo_t *cr = cairo_create(priv->hr.surface);
+    cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+    cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 0.0);
+    cairo_paint(cr);
+    cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+
+    cairo_select_font_face (cr, "Sans",
+                            CAIRO_FONT_SLANT_NORMAL,
+                            CAIRO_FONT_WEIGHT_BOLD);
+    cairo_set_font_size (cr, OSD_HR_FONT_SIZE);
+
+    char str[5];
+    if(priv->hr.rate > 0) 
+        snprintf(str, 4, "%03u", priv->hr.rate);
+    else if(priv->hr.rate == OSD_HR_INVALID) 
+        strcpy(str, "?");
+    else if(priv->hr.rate == OSD_HR_ERROR) 
+        strcpy(str, "Err");
+    else 
+        g_assert(priv->hr.rate != OSD_HR_NONE);
+
+    cairo_text_extents_t extents;
+    cairo_text_extents (cr, str, &extents);
+
+    cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+    cairo_set_line_width (cr, OSD_HR_FONT_SIZE/10);
+
+    cairo_move_to (cr, OSD_HR_W/2 - extents.width/2 + OSD_HR_FONT_SIZE/4, 
+                       OSD_HR_H/2 - extents.y_bearing/2);
+    cairo_text_path (cr, str);
+    cairo_stroke (cr);
+
+    cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+    cairo_move_to (cr, OSD_HR_W/2 - extents.width/2 + OSD_HR_FONT_SIZE/4, 
+                       OSD_HR_H/2 - extents.y_bearing/2);
+    cairo_show_text (cr, str);
+
+    osd_render_heart(cr, OSD_HR_W/2 - extents.width/2 - OSD_HR_FONT_SIZE/8, 
+                     OSD_HR_H/2, OSD_HR_FONT_SIZE/5, priv->hr.ok);
+    
+
+    cairo_destroy(cr);
+}
+
+void 
+osm_gps_map_osd_draw_hr (OsmGpsMap *map, gboolean ok, gint rate) {
+    g_return_if_fail (OSM_IS_GPS_MAP (map));
+
+    osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
+    g_return_if_fail (osd);
+
+    osd_priv_t *priv = (osd_priv_t*)osd->priv; 
+    g_return_if_fail (priv);
+
+    /* allocate heart rate surface */
+    if(rate != OSD_HR_NONE && !priv->hr.surface)
+        priv->hr.surface = 
+            cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 
+                                       OSD_HR_W, OSD_HR_H);
+
+    if(rate == OSD_HR_NONE && priv->hr.surface) {
+        cairo_surface_destroy(priv->hr.surface);
+        priv->hr.surface = NULL;
+    }
+
+    if(priv->hr.rate != rate || priv->hr.ok != ok) {
+        priv->hr.rate = rate;
+        priv->hr.ok = ok;
+        osd_render_hr(osd);
+    }
+    
+    osm_gps_map_redraw(map);
+}
+ 
+#endif // OSD_HEARTRATE
+ 
 static osd_button_t
 osd_check_int(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x, gint y) {
     osd_button_t but = OSD_NONE;
@@ -1961,6 +2171,17 @@ osd_draw(osm_gps_map_osd_t *osd, GdkDrawable *drawable)
     cairo_paint(cr);
 #endif
 
+#ifdef OSD_HEARTRATE
+    if(priv->hr.surface) {
+        x = (osd->widget->allocation.width - OSD_HR_W)/2;
+        y = OSD_HR_Y;
+        if(y < 0) y += osd->widget->allocation.height - OSD_COORDINATES_H;
+        
+        cairo_set_source_surface(cr, priv->hr.surface, x, y);
+        cairo_paint(cr);
+    }
+#endif
+
 #ifdef OSD_BALLOON
     if(priv->balloon.surface) {
  
@@ -2057,6 +2278,11 @@ osd_free(osm_gps_map_osd_t *osd)
          cairo_surface_destroy(priv->balloon.surface);
 #endif
 
+#ifdef OSD_HEARTRATE
+    if (priv->hr.surface) 
+         cairo_surface_destroy(priv->hr.surface);
+#endif
+
     g_free(priv);
     osd->priv = NULL;
 }
@@ -2109,6 +2335,10 @@ osm_gps_map_osd_classic_init(OsmGpsMap *map)
     priv->balloon.lon = OSM_GPS_MAP_INVALID;
 #endif
 
+#ifdef OSD_HEARTRATE
+    priv->hr.rate = OSD_HR_NONE;
+#endif
+
     osm_gps_map_register_osd(map, &osd_classic);
 }
 
